%
% cue combination experiment demo based on Landy/Kojima's orientation &
% contrast experiment
%
% response '1' for top stimulus appears left of bottom, '2' for right
%
% Note: subdirectory for subject must already exist
%
% Mike Landy - 8/2/12

function orientcontrexpt

oblur = 20;     % orientation blur in pixels (0, 10, 20, 40, 80)
cblur = 20;     % contrast blur in pixels (see stimulus list for
                % available combinations
deltaconds = [-15 0 15];    % delta-cue conditions
nshift = 9;     % number of levels
dshift = 4;     % number of pixels separating shift values
                % (centered on zero shift), must lead to even shift values
nrep = 30;       % total number of trials = nshift * #deltas * nrep
maxjitter = 12; % extra jitter up to +- maxjitter in pixels
fixsize = 11;   % should be odd and less than sep
sep = 31;       % separation between stimuli in pixels
scr = 2;        % screen to display on
gamma = 2.5;    % screen gamma to correct

% Experiment matrix columns
% 1: delta cond
% 2: delta value
% 3: shift cond
% 4: shift value
% 5: top/bottom - 1 means standard on top, 2 means standard on bottom
% 6: jitter value
% 7: topstd - top row extracted from standard image
% 8: leftstd - left column extracted from standard image
% 9: toptest - top row extracted from test image
% 10: lefttest - left column extracted from test image
% 11: response - 1 = 'test to right', 2 = 'standard to right'


rng('shuffle');
rstruct = mglResolution(scr);
vres = rstruct.screenHeight;
hres = rstruct.screenWidth;
if rem(dshift,2)
    fprintf(1,'dshift must be even\n');
end
if rem(nshift,2) == 2 && rem(dshift,4) ~= 0
    fprintf(1,'If nshift is even, dshift must be a multiple of 4\n');
    return;
end
minshift = - (nshift - 1)*dshift/2;
nconds = length(deltaconds);
ntrials = nconds*nshift*nrep;
subj = input('Subject initials? ','s');
blkno = input('Data block #? ');
ofile = [subj '/' subj num2str(blkno) '.mat'];

for i=1:nconds
    ifile = ['stimuli/oc.o0.90.' num2str(oblur) '.c50.100.' ...
        num2str(cblur) '.c' num2str(deltaconds(i)) '.mat'];
    load(ifile);
    imagelist{i} = img;
end

% need delta=0 for standard

i = find(deltaconds == 0);
if length(i > 0)
    imagezero = imagelist{i(1)};
else
    ifile = ['stimuli/oc.o0.90.' num2str(oblur) '.c50.100.' ...
        num2str(cblur) '.c0.mat'];
    load(ifile);
    imagezero = img;
end

% make trials list
ematrix = zeros(ntrials,11);
t = 0;
for i = 1:nconds
    for j = 1:nshift
        for k = 1:nrep
            t = t + 1;
            ematrix(t,1) = i;
            ematrix(t,2) = deltaconds(i);
            ematrix(t,3) = j;
            pos = minshift + dshift*(j-1);
            ematrix(t,4) = pos;
            ematrix(t,5) = randint(2);
            njit = randint(2*maxjitter+1) - (maxjitter+1);
            ematrix(t,6) = njit;
            ematrix(t,7) = 1 + randint(2048-400);
            ematrix(t,8) = 1 + (512-400)/2 + njit - pos/2;
            ematrix(t,9) = 1 + randint(2048-400);
            ematrix(t,10) = 1 + (512-400)/2 + njit + pos/2;
        end
    end
end
ematrix = ematrix(randperm(ntrials),:);

mglOpen(scr);
mglScreenCoordinates;
mglSetGammaTable(0,1,1/gamma,0,1,1/gamma,0,1,1/gamma);
mglClearScreen(.5);
mglDisplayCursor(0);
mglFlush;

fimg = 128*ones(fixsize);
fmid = 1 + floor(fixsize/2);
fimg(fmid,:) = 255;
fimg(:,fmid) = 255;
ftex = mglCreateTexture(fimg);
cimg = 128*ones(400);
ctex = mglCreateTexture(cimg);
vmid = floor(vres/2);
hmid = floor(hres/2);
o_ftop = vmid - fmid + 1;
o_fleft = hmid - fmid + 1;
o_utop = vmid - floor(sep/2) - 400;
o_ltop = o_utop + 400 + sep;
o_left = hmid - 200;
one = mglCharToKeycode({'1'});
queue = mglCharToKeycode({'q'});

fprintf(1,'Hit any key when ready\n');
while(~any(mglGetKeys))             % wait for keypress to begin
end

for t = 1:ntrials
    testimg = imagelist{ematrix(t,1)};
    pos = ematrix(t,4);
    testontop = ematrix(t,5) - 1;
    i_topstd = ematrix(t,7);
    i_leftstd = ematrix(t,8);
    i_toptest = ematrix(t,9);
    i_lefttest = ematrix(t,10);
    if testontop
        testtex = mglCreateTexture(double(testimg(i_toptest:(i_toptest+399), ...
            i_lefttest:(i_lefttest+399))));
        stdtex = mglCreateTexture(double(flipud(imagezero(i_topstd:(i_topstd+399), ...
            i_leftstd:(i_leftstd+399)))));
    else
        testtex = mglCreateTexture(double(flipud(testimg(i_toptest:(i_toptest+399), ...
            i_lefttest:(i_lefttest+399)))));
        stdtex = mglCreateTexture(double(imagezero(i_topstd:(i_topstd+399), ...
            i_leftstd:(i_leftstd+399))));
    end
    mglClearScreen(.5);
    mglBltTexture(ftex,[o_fleft,o_ftop],-1,-1);
    mglFlush;
    mglWaitSecs(.5);
    mglBltTexture(ftex,[o_fleft,o_ftop],-1,-1);
    if testontop
        mglBltTexture(testtex,[o_left,o_utop],-1,-1);
        mglBltTexture(stdtex,[o_left,o_ltop],-1,-1);
    else
        mglBltTexture(stdtex,[o_left,o_utop],-1,-1);
        mglBltTexture(testtex,[o_left,o_ltop],-1,-1);
    end
    mglFlush;
    mglDeleteTexture(testtex);
    mglDeleteTexture(stdtex);
    mglWaitSecs(.25);
    mglClearScreen(.5);
    mglBltTexture(ctex,[o_left,o_utop],-1,-1);
    mglBltTexture(ctex,[o_left,o_ltop],-1,-1);
    mglFlush;
    while(~any(mglGetKeys))
    end
    keys = mglGetKeys;
    if keys(queue)
        break;
    elseif testontop
        if keys(one)
            ematrix(t,11) = 2;
        else
            ematrix(t,11) = 1;
        end
    else
        if keys(one)
            ematrix(t,11) = 1;
        else
            ematrix(t,11) = 2;
        end
    end
end

mglClose;
results.ematrix = ematrix;
results.oblur = oblur;
results.cblur = cblur;
results.deltaconds = deltaconds;
results.nshift = nshift;
results.dshift = dshift;
results.nrep = nrep;
results.ntrials = ntrials;
results.maxjitter = maxjitter;
results.fixsize = fixsize;
results.sep = sep;
results.scr = scr;
results.gamma = gamma;
results.vres = vres;
results.hres = hres;
results.subj = subj;
results.blkno = blkno;

save(ofile,'results');

function i = randint(n)

i = 1 + floor(n*rand);

